import 'dart:async';
import 'dart:convert';

import 'package:archive/archive.dart';
import 'package:web_socket_channel/io.dart';

class HuobiKline {
  void Function(Map<String, dynamic> data) onData;
  IOWebSocketChannel channel;

  Future connect([String url = 'wss://api.huobi.pro/ws']) async {
    channel = IOWebSocketChannel.connect(url);
    channel.stream.listen((data) {
      Map<String, dynamic> json =
          jsonDecode(utf8.decode(GZipDecoder().decodeBytes(data)));
      if (json['ping'] != null) {
        send({'pong': json['ping']});
      } else {
        onData(json);
      }
    });
  }

  void send(Map<String, dynamic> data) {
    channel.sink.add(jsonEncode(data));
  }

  void subscribe(String symbol, String period) {
    send({'sub': 'market.$symbol.kline.$period'});
  }

  void query(String symbol, String period, DateTime from, [DateTime to]) {
    to ??= DateTime.now();
    send({
      'req': 'market.$symbol.kline.$period',
      'from': from.secondsSinceEpoch,
      'to': to.secondsSinceEpoch,
    });
  }
}

extension Seconds on DateTime {
  int get secondsSinceEpoch => millisecondsSinceEpoch ~/ 1000;
}
